#!/bin/sh
PackagePath="/var/packages/MailServer"
#config template
PostfixTemplate="$PackagePath/target/etc/template/main.template"
DevecotTemplate="$PackagePath/target/etc/template/dovecot.template"
AliasTemplate="$PackagePath/target/etc/template/aliases.template"
MasterTemplate="$PackagePath/target/etc/template/master.template"
#smtp
MailServerConf="$PackagePath/etc/mailserver.conf"
PostfixConf="/var/packages/MailServer/target/etc/main.cf"
SetRelayAuth=$PackagePath/target/bin/syno_set_relay_auth
GenConfValue=$PackagePath/target/bin/syno_gen_conf_value

#alias
AliasDB="$PackagePath/etc/alias.db"
Table="alias_info_table"
AliasConf="/var/packages/MailServer/target/etc/aliases" #need modify after package location done
GroupTable="/etc/group"
PostAlias=/var/packages/MailServer/target/sbin/postalias
#protocol
DovecotConf="/var/packages/MailServer/target/etc/dovecot.conf"

#spam filter
MasterConf="/var/packages/MailServer/target/etc/master.cf"

# save all user string
ALL_USERS=""

SMTP() {

	cat $PostfixTemplate > /tmp/tempinfo

	#smtp setting
	HostName=`/bin/get_key_value $MailServerConf smtp_hostname`
	if [ -z "$HostName" ]; then
		HostName="localhost"
	fi
	echo "myhostname = $HostName" >> /tmp/tempinfo

	SmtpAuth=`/bin/get_key_value $MailServerConf smtp_auth_enabled`
	if [ "$SmtpAuth" != "yes" ]; then
		SmtpAuth="no"
	fi
	echo "smtpd_sasl_auth_enable = $SmtpAuth" >> /tmp/tempinfo

	SmtpIgnoreLAN=`/bin/get_key_value $MailServerConf smtp_ignore_lan_auth`
	if [ "$SmtpAuth" = "yes" ] && [ "${SmtpIgnoreLAN}" = "no" ]; then
		echo "mynetworks = 127.0.0.0/8, [::1]/128" >> /tmp/tempinfo
	fi

	Domain=`/bin/get_key_value $MailServerConf canonical_domain`
	if [ -z  "$Domain" ]; then
		echo "mydestination = \$myhostname, localhost.\$mydomain, localhost" >> /tmp/tempinfo
	else
		echo "mydestination = \$myhostname, localhost.\$mydomain, localhost, $Domain" >> /tmp/tempinfo
	fi

	MessageSizeLimit=`/bin/get_key_value $MailServerConf message_limit`
	if [ -z "$MessageSizeLimit" ]; then
		MessageSizeLimit=10240000
	else
		MessageSizeLimit=`/usr/bin/expr $MessageSizeLimit \* 1048576`
	fi
	echo "message_size_limit = $MessageSizeLimit" >> /tmp/tempinfo

	SmtpEnable=`/bin/get_key_value $MailServerConf smtp_enabled`

	# SMTP-SSL
	SmtpsEnable=`/bin/get_key_value $MailServerConf smtp_ssl_enabled`
	if [ "$SmtpEnable" = "yes" ] && [ "$SmtpsEnable" = "yes" ]; then
		echo "smtpd_use_tls = yes" >> /tmp/tempinfo
		echo "tls_random_source = dev:/dev/urandom" >> /tmp/tempinfo
	fi

	#relay setting
	RelayEnable=`/bin/get_key_value $MailServerConf smtp_relay_enabled`
	if [ "$SmtpEnable" = "yes" ] && [ "$RelayEnable" = "yes" ]; then
		RelayServer=`/bin/get_key_value $MailServerConf smtp_relay_server`
		RelayPort=`/bin/get_key_value $MailServerConf smtp_relay_server_port`

		if [ ! -z "$RelayServer" ] && [ ! -z "$RelayPort" ]; then
			echo "relayhost = ${RelayServer}:${RelayPort}" >> /tmp/tempinfo
			RelayAuth=`/bin/get_key_value $MailServerConf smtp_relay_auth_enabled`
			RelayTLS=`/bin/get_key_value $MailServerConf smtp_relay_secure_conn`
			if [ "$RelayTLS" == "yes" ]; then
				echo "smtp_use_tls = $RelayTLS" >> /tmp/tempinfo
				echo "smtp_sasl_tls_security_options = noanonymous" >> /tmp/tempinfo
			fi

			if [ "$RelayAuth" == "yes" ]; then
				echo "smtp_sasl_auth_enable = $RelayAuth" >> /tmp/tempinfo
				echo "smtp_sasl_password_maps = hash:/var/packages/MailServer/target/etc/sasl_passwd" >> /tmp/tempinfo
				echo "smtp_sasl_security_options = noanonymous" >> /tmp/tempinfo
				$SetRelayAuth
			fi
		fi
	fi

	#set spam-filter: DNSBL
        SmtpdRecipientRestriction="permit_mynetworks, permit_sasl_authenticated, reject_unauth_destination"
	DNSBLEnable=`/bin/get_key_value $MailServerConf spam_dnsbl_enable`
	if [ "$DNSBLEnable" == "yes" ]; then
		DNSBLServer=`$GenConfValue dnsbl`
		if [ ! -z "${DNSBLServer}" ]; then
			SmtpdRecipientRestriction="${SmtpdRecipientRestriction}, ${DNSBLServer}"
		fi
	fi
	echo "smtpd_recipient_restrictions = ${SmtpdRecipientRestriction}" >> /tmp/tempinfo

	/bin/mv /tmp/tempinfo $PostfixConf

}

Alias() {

	if [ ! -e $AliasDB ]; then
		echo "using alias template"
                /bin/cp $AliasTemplate $AliasConf
                /bin/chmod 644 $AliasConf
                $PostAlias $AliasConf
		exit 0
	fi

	cat $AliasTemplate > /tmp/tempinfo
	sqlite3 $AliasDB "SELECT DISTINCT alias FROM $Table" | while read alias
	do
		AliasMember=
		sqlite3 $AliasDB "SELECT member , type FROM $Table WHERE alias='$alias'" | while read info
		do
			member=`echo $info | cut -d '|' -f1`
			type=`echo $info | cut -d '|' -f2`
			if [ $type = 1 ]; then
				if [ "$member" = "users" ]; then
					GetAllUsers
					AliasMember=$AliasMember","$ALL_USERS
				else
					GroupMember=`grep $member $GroupTable | cut -d ':' -f4`
					AliasMember=$AliasMember","$GroupMember
				fi
			else
				AliasMember=$AliasMember","$member
			fi
			echo $AliasMember > /tmp/aliastemp
		done
		AliasMember=`cat /tmp/aliastemp | sed 's/.\(.*\)/\1/'` #remove first character ","
		echo "$alias : $AliasMember" >> /tmp/tempinfo

	done
	rm -rf /tmp/aliastemp
	/bin/mv /tmp/tempinfo $AliasConf
	$PostAlias $AliasConf
}

Protocol() {

	cat $DevecotTemplate > /tmp/tempinfo
	for key in imap imaps pop3 pop3s; do
		Enabled=`/bin/get_key_value $MailServerConf ${key}_enabled`
		if [ "$Enabled" = "yes" ]; then
			Protocols="${Protocols} ${key}"
			if [ "${key}" = "imap" ]; then
				Imap="imap"
			fi
		fi
	done

	Imaps=`echo ${Protocols} | grep imaps`
	Pop3Pop3s=`echo ${Protocols} | grep pop3`

	ListenValue="*, [::]"
	if [ -z "$Imap" ]; then
		Protocols="imap${Protocols}"
		ListenValue="localhost"
	fi

	echo "protocols = $Protocols" >> /tmp/tempinfo

	echo "protocol imap {" >> /tmp/tempinfo
	echo "     listen = $ListenValue" >> /tmp/tempinfo
	if [ "$Imaps" ]; then
		echo "     ssl_listen = *, [::]" >> /tmp/tempinfo
	fi
	echo "}" >> /tmp/tempinfo

	if [ "$Pop3Pop3s" ]; then
		echo "protocol pop3 {" >> /tmp/tempinfo
		echo "     listen = *, [::]" >> /tmp/tempinfo
		echo "     ssl_listen = *, [::]" >> /tmp/tempinfo
		echo "}" >> /tmp/tempinfo
	fi

	/bin/mv /tmp/tempinfo $DovecotConf
}

MasterCf() {
	cat $MasterTemplate > /tmp/tempinfo

	# SMTP-SSL
	local SmtpEnable=`/bin/get_key_value $MailServerConf smtp_enabled`
	local SmtpsEnable=`/bin/get_key_value $MailServerConf smtp_ssl_enabled`
	if [ "$SmtpEnable" = "yes" ] && [ "$SmtpsEnable" = "yes" ]; then
		echo "smtps inet n - n - - smtpd -o smtpd_tls_wrappermode=yes -o smtpd_sasl_auth_enable=yes" >> /tmp/tempinfo
	fi

	SpamEnable=`/bin/get_key_value $MailServerConf spam_enable`
	if [ "$SpamEnable" = "yes" ]; then
		echo "smtp inet n - n - - smtpd -o content_filter=spamassassin" >> /tmp/tempinfo
		echo "spamassassin unix - n n - - pipe user=spamfilter argv=/var/packages/MailServer/target/bin/filter.sh -oi -f \${sender} \${recipient}" >> /tmp/tempinfo
	else
		echo "smtp inet n - n - - smtpd" >> /tmp/tempinfo
	fi
	/bin/mv /tmp/tempinfo $MasterConf

}

GetAllUsers() {
    synouser --enum local | sed '1d' | while read user
    do
        ALL_USERS=${ALL_USERS}","${user}
        echo ${ALL_USERS} > /tmp/user.all
    done
    ALL_USERS=`cat /tmp/user.all | sed 's/.\(.*\)/\1/'`
    rm -rf /tmp/user.all
}

case "$1" in
	smtp)
		SMTP
		MasterCf
		;;
	protocol)
		Protocol
		;;
	alias)
		Alias
		;;
    spam)
		MasterCf
		;;
	*)
		echo "Usage: $0 {smtp|protocol|alias|spam}" >&2
		exit 1
		;;
esac
