#!/bin/sh

SASLAUTHD=/var/packages/MailServer/target/sbin/saslauthd
POSTFIX=/var/packages/MailServer/target/sbin/postfix

PackagePath="/var/packages/MailServer"
MailServerConf="$PackagePath/etc/mailserver.conf"
PID_SPAMD=/var/run/spamd.pid
SPAMD=/var/packages/MailServer/target/bin/spamd
SpamEnable=`/bin/get_key_value $MailServerConf spam_enable`

LogDaemon=/var/packages/MailServer/target/sbin/syno_mailloggerd
LogDaemonPID=/var/run/syno_mailloggerd.pid

start() {
	echo "Starting Maillog Daemon..."
	${LogDaemon}

	echo "Starting SASLAuthd..."
	$SASLAUTHD -a shadow

	echo "Starting Postfix..."
	$POSTFIX start

	if [ "x${SpamEnable}" = "xyes" ]; then
		echo "Starting SPAMD..."
		${SPAMD} -d -r ${PID_SPAMD}
	fi
}

stop() {
	saslauthd_pid=/var/run/saslauthd/saslauthd.pid
	if [ -f $saslauthd_pid ]; then
		echo "Stopping SASLAuthd..."
		kill `cat $saslauthd_pid`
	fi
	echo "Stopping Postfix..."
	$POSTFIX stop

	if [ -f $PID_SPAMD ]; then
		echo "Stopping SPAMD..."
		kill `cat ${PID_SPAMD}`
	fi

    echo "Stop Log Daemon..."
    kill `cat ${LogDaemonPID}`
}

restart() {

	#use reload instead of (stop => sleep(1) => start), because reload is faster

	#if spamassassin is just enable, start daemion
	if [ "$SpamEnable" = "yes" ] && [ ! -f $PID_SPAMD ]; then
		echo "Starting SPAMD..."
		${SPAMD} -d -r ${PID_SPAMD}
	elif [ "$SpamEnable" = "no" ] && [ -f $PID_SPAMD ]; then
		echo "Stopping SPAMD..."
		kill `cat ${PID_SPAMD}`
	fi

	echo "Reload Postfix..."
	$POSTFIX reload
}

case "$1" in
	start)
		start
		;;
	stop)
		stop
		;;
	restart)
		restart
		;;
	*)
		echo "Usage: $0 {start|stop|reload}" >&2
		exit 1
		;;
esac

exit 0
